/*************************************************************************************************
 *
 * Modeling and animation (TNM079) 2007
 * Code base for lab assignments. Copyright:
 *   Gunnar Johansson (gunnar.johansson@itn.liu.se)
 *   Ken Museth (ken.museth@itn.liu.se)
 *   Michael Bang Nielsen (bang@daimi.au.dk)
 *   Ola Nilsson (ola.nilsson@itn.liu.se)
 *   Andreas Sderstrm (andreas.soderstrom@itn.liu.se)
 *
 *************************************************************************************************/
#ifndef __HALF_EDGE_H__
#define __HALF_EDGE_H__

#include "Mesh.h"
#include <map>
#include <set>
#include <queue>

class HalfEdgeMesh : public Mesh 
{
public:
  HalfEdgeMesh();
  ~HalfEdgeMesh();
	
  //! Adds a triangle to the mesh. \sa addTriangle
  virtual bool addTriangle(const Vector3<float> &v1, const Vector3<float> &v2, const Vector3<float> & v3);


  virtual float area() const;

  virtual float volume() const;

  virtual int genus() const;

  virtual float curvature(const unsigned int vertexIndex, const Vector3<float>& n);


  //caculate the face normals and store the 
  //resulting normal inside Face structure
  virtual void calculateFaceNormals();

  virtual void calculateVertexNormals();

  virtual void draw();

  //validate the mesh
  void validate();

  //merge all  the boundaries of the mesh
  void mergeAllBoundaries();

protected:

  //! References a border, only for face pointers
  const static unsigned int BORDER;
  //! Denotes a reference to a non-existing object
  const static unsigned int UNINITIALIZED;

  struct HalfEdge 
  {
    HalfEdge() : vert(UNINITIALIZED), face(UNINITIALIZED), next(UNINITIALIZED), prev(UNINITIALIZED), pair(UNINITIALIZED) { }
    unsigned int vert;  // index into mVerts
    unsigned int face;   // index into mFaces
    unsigned int next;  // index into mEdges
    unsigned int prev;  // index into mEdges
    unsigned int pair;  // index into mEdges
  };

  struct Vertex 
  {
    Vertex() : edge(UNINITIALIZED) { }
    Vector3<float> vec;  // the vertex coordinates
	 Vector3<float> v_normal; // vertex normal
    unsigned int edge; // index into mEdges
  };

  struct Face 
  {
    Face() : edge(UNINITIALIZED) { }
    unsigned int edge; // index into mEdges
    Vector3<float> normal; // the face normal
  };

  std::vector<HalfEdge> mEdges;
  std::vector<Vertex> mVerts;
  std::vector<Face> mFaces;
  std::vector<float> mCurvature;

  std::map<Vector3<float>, unsigned int> mUniqueVerts;

  //! Adds a vertex to the mesh
  virtual bool addVertex(const Vector3<float>& v, unsigned int &indx);

  bool addHalfEdgePair(unsigned int v1, unsigned int v2, unsigned int &indx1, unsigned int &indx2);


  //merge an edge into  the boundary of the mesh
  void mergeBoundaryEdge(unsigned int indx);

  
  // Given a vertex, find all triangles that includes this vertex
  virtual bool findNeighbourTriangles(const unsigned int vertexIndex, std::vector<unsigned int>& foundTriangles) const;

};

#endif
